@@ -4,9 +4,10 @@
 #
 # Fetch one or more remote refs and merge it/them into the current HEAD.
 
-USAGE='[-n | --no-summary] [--no-commit] [-s strategy]... [<fetch-options>] <repo> <head>...'
+USAGE='[-n | --no-summary] [--[no-]commit] [--[no-]squash] [--[no-]ff] [-s strategy]... [<fetch-options>] <repo> <head>...'
 LONG_USAGE='Fetch one or more remote refs and merge it/them into the current HEAD.'
 SUBDIRECTORY_OK=Yes
+OPTIONS_SPEC=
 . git-sh-setup
 set_reflog_action "pull $*"
 require_work_tree
@@ -15,7 +16,10 @@ cd_to_toplevel
 test -z "$(git ls-files -u)" ||
 	die "You are in the middle of a conflicted merge."
 
-strategy_args= no_summary= no_commit= squash=
+strategy_args= no_summary= no_commit= squash= no_ff=
+curr_branch=$(git symbolic-ref -q HEAD)
+curr_branch_short=$(echo "$curr_branch" | sed "s|refs/heads/||")
+rebase=$(git config --bool branch.$curr_branch_short.rebase)
 while :
 do
 	case "$1" in
@@ -27,8 +31,16 @@ do
 		;;
 	--no-c|--no-co|--no-com|--no-comm|--no-commi|--no-commit)
 		no_commit=--no-commit ;;
+	--c|--co|--com|--comm|--commi|--commit)
+		no_commit=--commit ;;
 	--sq|--squ|--squa|--squas|--squash)
 		squash=--squash ;;
+	--no-sq|--no-squ|--no-squa|--no-squas|--no-squash)
+		squash=--no-squash ;;
+	--ff)
+		no_ff=--ff ;;
+	--no-ff)
+		no_ff=--no-ff ;;
 	-s=*|--s=*|--st=*|--str=*|--stra=*|--strat=*|--strate=*|\
 		--strateg=*|--strategy=*|\
 	-s|--s|--st|--str|--stra|--strat|--strate|--strateg|--strategy)
@@ -43,6 +55,12 @@ do
 		esac
 		strategy_args="${strategy_args}-s $strategy "
 		;;
+	-r|--r|--re|--reb|--reba|--rebas|--rebase)
+		rebase=true
+		;;
+	--no-r|--no-re|--no-reb|--no-reba|--no-rebas|--no-rebase)
+		rebase=false
+		;;
 	-h|--h|--he|--hel|--help)
 		usage
 		;;
@@ -54,6 +72,49 @@ do
 	shift
 done
 
+error_on_no_merge_candidates () {
+	exec >&2
+	for opt
+	do
+		case "$opt" in
+		-t|--t|--ta|--tag|--tags)
+			echo "Fetching tags only, you probably meant:"
+			echo "  git fetch --tags"
+			exit 1
+		esac
+	done
+
+	curr_branch=${curr_branch#refs/heads/}
+
+	echo "You asked me to pull without telling me which branch you"
+	echo "want to merge with, and 'branch.${curr_branch}.merge' in"
+	echo "your configuration file does not tell me either.  Please"
+	echo "name which branch you want to merge on the command line and"
+	echo "try again (e.g. 'git pull <repository> <refspec>')."
+	echo "See git-pull(1) for details on the refspec."
+	echo
+	echo "If you often merge with the same branch, you may want to"
+	echo "configure the following variables in your configuration"
+	echo "file:"
+	echo
+	echo "    branch.${curr_branch}.remote = <nickname>"
+	echo "    branch.${curr_branch}.merge = <remote-ref>"
+	echo "    remote.<nickname>.url = <url>"
+	echo "    remote.<nickname>.fetch = <refspec>"
+	echo
+	echo "See git-config(1) for details."
+	exit 1
+}
+
+test true = "$rebase" && {
+	. git-parse-remote &&
+	origin="$1"
+	test -z "$origin" && origin=$(get_default_remote)
+	reflist="$(get_remote_refs_for_fetch "$@" 2>/dev/null |
+		sed "s|refs/heads/\(.*\):|\1|")" &&
+	oldremoteref="$(git rev-parse --verify \
+		"refs/remotes/$origin/$reflist" 2>/dev/null)"
+}
 orig_head=$(git rev-parse --verify HEAD 2>/dev/null)
 git-fetch --update-head-ok "$@" || exit 1
 
@@ -86,35 +147,14 @@ merge_head=$(sed -e '/	not-for-merge	/d' \
 
 case "$merge_head" in
 '')
-	curr_branch=$(git symbolic-ref -q HEAD)
 	case $? in
-	  0) ;;
-	  1) echo >&2 "You are not currently on a branch; you must explicitly"
-	     echo >&2 "specify which branch you wish to merge:"
-	     echo >&2 "  git pull <remote> <branch>"
-	     exit 1;;
-	  *) exit $?;;
+	0) error_on_no_merge_candidates "$@";;
+	1) echo >&2 "You are not currently on a branch; you must explicitly"
+	   echo >&2 "specify which branch you wish to merge:"
+	   echo >&2 "  git pull <remote> <branch>"
+	   exit 1;;
+	*) exit $?;;
 	esac
-	curr_branch=${curr_branch#refs/heads/}
-
-	echo >&2 "You asked me to pull without telling me which branch you"
-	echo >&2 "want to merge with, and 'branch.${curr_branch}.merge' in"
-	echo >&2 "your configuration file does not tell me either.  Please"
-	echo >&2 "name which branch you want to merge on the command line and"
-	echo >&2 "try again (e.g. 'git pull <repository> <refspec>')."
-	echo >&2 "See git-pull(1) for details on the refspec."
-	echo >&2
-	echo >&2 "If you often merge with the same branch, you may want to"
-	echo >&2 "configure the following variables in your configuration"
-	echo >&2 "file:"
-	echo >&2
-	echo >&2 "    branch.${curr_branch}.remote = <nickname>"
-	echo >&2 "    branch.${curr_branch}.merge = <remote-ref>"
-	echo >&2 "    remote.<nickname>.url = <url>"
-	echo >&2 "    remote.<nickname>.fetch = <refspec>"
-	echo >&2
-	echo >&2 "See git-config(1) for details."
-	exit 1
 	;;
 ?*' '?*)
 	if test -z "$orig_head"
@@ -133,5 +173,7 @@ then
 fi
 
 merge_name=$(git fmt-merge-msg <"$GIT_DIR/FETCH_HEAD") || exit
-exec git-merge $no_summary $no_commit $squash $strategy_args \
+test true = "$rebase" &&
+	exec git-rebase --onto $merge_head ${oldremoteref:-$merge_head}
+exec git-merge $no_summary $no_commit $squash $no_ff $strategy_args \
 	"$merge_name" HEAD $merge_head
