@@ -7,7 +7,7 @@
 #include "cache.h"
 #include "blob.h"
 
-static void hash_object(const char *path, const char *type, int write_object)
+static void hash_object(const char *path, enum object_type type, int write_object)
 {
 	int fd;
 	struct stat st;
@@ -15,7 +15,7 @@ static void hash_object(const char *path, const char *type, int write_object)
 	fd = open(path, O_RDONLY);
 	if (fd < 0 ||
 	    fstat(fd, &st) < 0 ||
-	    index_fd(sha1, fd, &st, write_object, type))
+	    index_fd(sha1, fd, &st, write_object, type, path))
 		die(write_object
 		    ? "Unable to add %s to database"
 		    : "Unable to hash %s", path);
@@ -42,11 +42,13 @@ int main(int argc, char **argv)
 	int prefix_length = -1;
 	int no_more_flags = 0;
 
+	git_config(git_default_config);
+
 	for (i = 1 ; i < argc; i++) {
 		if (!no_more_flags && argv[i][0] == '-') {
 			if (!strcmp(argv[i], "-t")) {
 				if (argc <= ++i)
-					die(hash_object_usage);
+					usage(hash_object_usage);
 				type = argv[i];
 			}
 			else if (!strcmp(argv[i], "-w")) {
@@ -66,14 +68,14 @@ int main(int argc, char **argv)
 				hash_stdin(type, write_object);
 			}
 			else
-				die(hash_object_usage);
-		} 
+				usage(hash_object_usage);
+		}
 		else {
 			const char *arg = argv[i];
 			if (0 <= prefix_length)
 				arg = prefix_filename(prefix, prefix_length,
 						      arg);
-			hash_object(arg, type, write_object);
+			hash_object(arg, type_from_string(type), write_object);
 			no_more_flags = 1;
 		}
 	}
